<?php

/* ***********************************************************
 * This file was automatically generated on 2012-05-24.      *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generator git on tinkerforge.com                   *
 *************************************************************/

namespace Tinkerforge;

require_once(__DIR__ . '/IPConnection.php');

/**
 * Device for controlling Stacks and four Bricklets
 */
class BrickMaster extends Device
{


    /**
     * @internal
     */
    const FUNCTION_GET_STACK_VOLTAGE = 1;

    /**
     * @internal
     */
    const FUNCTION_GET_STACK_CURRENT = 2;

    /**
     * @internal
     */
    const FUNCTION_SET_EXTENSION_TYPE = 3;

    /**
     * @internal
     */
    const FUNCTION_GET_EXTENSION_TYPE = 4;

    /**
     * @internal
     */
    const FUNCTION_IS_CHIBI_PRESENT = 5;

    /**
     * @internal
     */
    const FUNCTION_SET_CHIBI_ADDRESS = 6;

    /**
     * @internal
     */
    const FUNCTION_GET_CHIBI_ADDRESS = 7;

    /**
     * @internal
     */
    const FUNCTION_SET_CHIBI_MASTER_ADDRESS = 8;

    /**
     * @internal
     */
    const FUNCTION_GET_CHIBI_MASTER_ADDRESS = 9;

    /**
     * @internal
     */
    const FUNCTION_SET_CHIBI_SLAVE_ADDRESS = 10;

    /**
     * @internal
     */
    const FUNCTION_GET_CHIBI_SLAVE_ADDRESS = 11;

    /**
     * @internal
     */
    const FUNCTION_GET_CHIBI_SIGNAL_STRENGTH = 12;

    /**
     * @internal
     */
    const FUNCTION_GET_CHIBI_ERROR_LOG = 13;

    /**
     * @internal
     */
    const FUNCTION_SET_CHIBI_FREQUENCY = 14;

    /**
     * @internal
     */
    const FUNCTION_GET_CHIBI_FREQUENCY = 15;

    /**
     * @internal
     */
    const FUNCTION_SET_CHIBI_CHANNEL = 16;

    /**
     * @internal
     */
    const FUNCTION_GET_CHIBI_CHANNEL = 17;

    /**
     * Creates an object with the unique device ID $uid. This object can
     * then be added to the IP connection.
     *
     * @param string $uid
     */
    public function __construct($uid)
    {
        parent::__construct($uid);

        $this->expectedName = 'Master Brick';

        $this->bindingVersion = array(1, 1, 0);

    }

    /**
     * @internal
     * @param string $header
     * @param string $data
     */
    public function handleCallback($header, $data)
    {
        call_user_func(array($this, $this->callbackWrappers[$header['functionID']]), $data);
    }

    /**
     * Returns the stack voltage in mV. The stack voltage is the
     * voltage that is supplied via the stack, i.e. it is given by a 
     * Step-Down or Step-Up Power Supply.
     * 
     * 
     * @return int
     */
    public function getStackVoltage()
    {
        $payload = '';


        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_STACK_VOLTAGE, $payload, 2);


        $payload = unpack('v1voltage', $data);

        return $payload['voltage'];
    }

    /**
     * Returns the stack current in mA. The stack current is the
     * current that is drawn via the stack, i.e. it is given by a
     * Step-Down or Step-Up Power Supply.
     * 
     * 
     * @return int
     */
    public function getStackCurrent()
    {
        $payload = '';


        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_STACK_CURRENT, $payload, 2);


        $payload = unpack('v1current', $data);

        return $payload['current'];
    }

    /**
     * Writes the extension type to the EEPROM of a specified extension. 
     * The extension is either 0 or 1 (0 is the on the bottom, 1 is the on on top, 
     * if only one extension is present use 0).
     * 
     * Possible extension types:
     * 
     * <code>
     *  "Type", "Description"
     * 
     *  "1",    "Chibi"
     *  "2",    "RS485"
     * </code>
     * 
     * The extension type is already set when bought and it can be set with the 
     * Brick Viewer, it is unlikely that you need this function.
     * 
     * The value will be saved in the EEPROM of the Chibi Extension, it does not
     * have to be set on every startup.
     * 
     * @param int $extension
     * @param int $exttype
     * 
     * @return void
     */
    public function setExtensionType($extension, $exttype)
    {
        $payload = '';
        $payload .= pack('C', $extension);
        $payload .= pack('V', $exttype);

        $this->sendRequestNoResponse(self::FUNCTION_SET_EXTENSION_TYPE, $payload);





    }

    /**
     * Returns the extension type for a given extension as set by 
     * BrickMaster::setExtensionType().
     * 
     * @param int $extension
     * 
     * @return int
     */
    public function getExtensionType($extension)
    {
        $payload = '';
        $payload .= pack('C', $extension);

        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_EXTENSION_TYPE, $payload, 4);


        $payload = unpack('V1exttype', $data);

        return IPConnection::fixUnpackedUInt32($payload['exttype']);
    }

    /**
     * Returns true if a Chibi Extension is available to be used by the Master.
     * 
     * 
     * @return bool
     */
    public function isChibiPresent()
    {
        $payload = '';


        $data = $this->sendRequestExpectResponse(self::FUNCTION_IS_CHIBI_PRESENT, $payload, 1);


        $payload = unpack('C1present', $data);

        return (bool)$payload['present'];
    }

    /**
     * Sets the address (1-255) belonging to the Chibi Extension.
     * 
     * It is possible to set the address with the Brick Viewer and it will be 
     * saved in the EEPROM of the Chibi Extension, it does not
     * have to be set on every startup.
     * 
     * @param int $address
     * 
     * @return void
     */
    public function setChibiAddress($address)
    {
        $payload = '';
        $payload .= pack('C', $address);

        $this->sendRequestNoResponse(self::FUNCTION_SET_CHIBI_ADDRESS, $payload);





    }

    /**
     * Returns the address as set by BrickMaster::setChibiAddress().
     * 
     * 
     * @return int
     */
    public function getChibiAddress()
    {
        $payload = '';


        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_CHIBI_ADDRESS, $payload, 1);


        $payload = unpack('C1address', $data);

        return $payload['address'];
    }

    /**
     * Sets the address (1-255) of the Chibi Master. This address is used if the
     * Chibi Extension is used as slave (i.e. it does not have a USB connection).
     * 
     * It is possible to set the address with the Brick Viewer and it will be 
     * saved in the EEPROM of the Chibi Extension, it does not
     * have to be set on every startup.
     * 
     * @param int $address
     * 
     * @return void
     */
    public function setChibiMasterAddress($address)
    {
        $payload = '';
        $payload .= pack('C', $address);

        $this->sendRequestNoResponse(self::FUNCTION_SET_CHIBI_MASTER_ADDRESS, $payload);





    }

    /**
     * Returns the address as set by BrickMaster::setChibiMasterAddress().
     * 
     * 
     * @return int
     */
    public function getChibiMasterAddress()
    {
        $payload = '';


        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_CHIBI_MASTER_ADDRESS, $payload, 1);


        $payload = unpack('C1address', $data);

        return $payload['address'];
    }

    /**
     * Sets up to 256 slave addresses. The address numeration has to be used 
     * ascending from 0. For example: If you use the Chibi Extension in Master mode
     * (i.e. the stack has an USB connection) and you want to talk to three other
     * Chibi stacks with the IDs 17, 23, and 42, you should call with "(0, 17),
     * (1, 23) and (2, 42)".
     * 
     * It is possible to set the addresses with the Brick Viewer and it will be 
     * saved in the EEPROM of the Chibi Extension, they don't
     * have to be set on every startup.
     * 
     * @param int $num
     * @param int $address
     * 
     * @return void
     */
    public function setChibiSlaveAddress($num, $address)
    {
        $payload = '';
        $payload .= pack('C', $num);
        $payload .= pack('C', $address);

        $this->sendRequestNoResponse(self::FUNCTION_SET_CHIBI_SLAVE_ADDRESS, $payload);





    }

    /**
     * Returns the slave address for a given num as set by 
     * BrickMaster::setChibiSlaveAddress().
     * 
     * @param int $num
     * 
     * @return int
     */
    public function getChibiSlaveAddress($num)
    {
        $payload = '';
        $payload .= pack('C', $num);

        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_CHIBI_SLAVE_ADDRESS, $payload, 1);


        $payload = unpack('C1address', $data);

        return $payload['address'];
    }

    /**
     * Returns the signal strength in dBm. The signal strength updates every time a
     * packet is received.
     * 
     * 
     * @return int
     */
    public function getChibiSignalStrength()
    {
        $payload = '';


        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_CHIBI_SIGNAL_STRENGTH, $payload, 1);


        $payload = unpack('C1signal_strength', $data);

        return $payload['signal_strength'];
    }

    /**
     * Returns underrun, CRC error, no ACK and overflow error counts of the Chibi
     * communication. If these errors start rising, it is likely that either the
     * distance between two Chibi stacks is becoming too big or there are
     * interferences.
     * 
     * 
     * @return array
     */
    public function getChibiErrorLog()
    {
        $result = array();

        $payload = '';


        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_CHIBI_ERROR_LOG, $payload, 8);


        $payload = unpack('v1underrun/v1crc_error/v1no_ack/v1overflow', $data);

        $result['underrun'] = $payload['underrun'];
        $result['crc_error'] = $payload['crc_error'];
        $result['no_ack'] = $payload['no_ack'];
        $result['overflow'] = $payload['overflow'];

        return $result;
    }

    /**
     * Sets the Chibi frequency range for the Chibi Extension. Possible values are:
     * 
     * <code>
     *  "Type", "Description"
     * 
     *  "0",    "OQPSK 868Mhz (Europe)"
     *  "1",    "OQPSK 915Mhz (US)"
     *  "2",    "OQPSK 780Mhz (China)"
     *  "3",    "BPSK40 915Mhz"
     * </code>
     * 
     * It is possible to set the frequency with the Brick Viewer and it will be 
     * saved in the EEPROM of the Chibi Extension, it does not
     * have to be set on every startup.
     * 
     * @param int $frequency
     * 
     * @return void
     */
    public function setChibiFrequency($frequency)
    {
        $payload = '';
        $payload .= pack('C', $frequency);

        $this->sendRequestNoResponse(self::FUNCTION_SET_CHIBI_FREQUENCY, $payload);





    }

    /**
     * Returns the frequency value as set by BrickMaster::setChibiFrequency().
     * 
     * 
     * @return int
     */
    public function getChibiFrequency()
    {
        $payload = '';


        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_CHIBI_FREQUENCY, $payload, 1);


        $payload = unpack('C1frequency', $data);

        return $payload['frequency'];
    }

    /**
     * Sets the channel used by the Chibi Extension. Possible channels are
     * different for different frequencies:
     * 
     * <code>
     *  "Frequency",             "Possible Channels"
     * 
     *  "OQPSK 868Mhz (Europe)", "0"
     *  "OQPSK 915Mhz (US)",     "1, 2, 3, 4, 5, 6, 7, 8, 9, 10"
     *  "OQPSK 780Mhz (China)",  "0, 1, 2, 3"
     *  "BPSK40 915Mhz",         "1, 2, 3, 4, 5, 6, 7, 8, 9, 10"
     * </code>
     * 
     * It is possible to set the frequency with the Brick Viewer and it will be 
     * saved in the EEPROM of the Chibi Extension, it does not
     * have to be set on every startup.
     * 
     * @param int $channel
     * 
     * @return void
     */
    public function setChibiChannel($channel)
    {
        $payload = '';
        $payload .= pack('C', $channel);

        $this->sendRequestNoResponse(self::FUNCTION_SET_CHIBI_CHANNEL, $payload);





    }

    /**
     * Returns the channel as set by BrickMaster::setChibiChannel().
     * 
     * 
     * @return int
     */
    public function getChibiChannel()
    {
        $payload = '';


        $data = $this->sendRequestExpectResponse(self::FUNCTION_GET_CHIBI_CHANNEL, $payload, 1);


        $payload = unpack('C1channel', $data);

        return $payload['channel'];
    }
}

?>
